@@ -1,14 +1,14 @@
 validate_X <- function(X, allow.na = FALSE) {
-  valid.classes <- c("matrix", "data.frame", "dgCMatrix")
+  valid.classes <- c("matrix", "data.frame")
 
   if (!inherits(X, valid.classes)) {
     stop(paste(
       "Currently the only supported data input types are:",
-      "`matrix`, `data.frame`, `dgCMatrix`"
+      "`matrix`, `data.frame`"
     ))
   }
 
-  if (!inherits(X, "dgCMatrix") && !is.numeric(as.matrix(X))) {
+  if (!is.numeric(as.matrix(X))) {
     stop(paste(
       "The feature matrix X must be numeric. GRF does not",
       "currently support non-numeric features. If factor variables",
@@ -21,7 +21,7 @@ validate_X <- function(X, allow.na = FALSE) {
     ))
   }
 
-  has.missing.values <- any(is.na(X))
+  has.missing.values <- anyNA(X)
 
   if (!allow.na && has.missing.values) {
     stop("The feature matrix X contains at least one NA.")
@@ -30,11 +30,19 @@ validate_X <- function(X, allow.na = FALSE) {
   has.missing.values
 }
 
-validate_observations <- function(V, X) {
-  if (is.matrix(V) && ncol(V) == 1) {
-    V <- as.vector(V)
-  } else if (!is.vector(V)) {
-    stop(paste("Observations (W, Y, Z or D) must be vectors."))
+validate_observations <- function(V, X, allow.matrix = FALSE) {
+  if (!allow.matrix) {
+    if (is.matrix(V) && ncol(V) == 1) {
+      V <- as.vector(V)
+    } else if (!is.vector(V)) {
+      stop("Observations (W, Y, Z or D) must be vectors.")
+    }
+  } else {
+    if (is.matrix(V) || is.data.frame(V) || is.vector(V)) {
+      V <- as.matrix(V)
+    } else {
+      stop("Observations Y must be either a vector/matrix/data.frame.")
+    }
   }
 
   if (!is.numeric(V) && !is.logical(V)) {
@@ -44,11 +52,11 @@ validate_observations <- function(V, X) {
     ))
   }
 
-  if (any(is.na(V))) {
+  if (anyNA(V)) {
     stop("The vector of observations (W, Y, Z or D) contains at least one NA.")
   }
 
-  if (length(V) != nrow(X)) {
+  if (NROW(V) != nrow(X)) {
     stop("length of observation (W, Y, Z or D) does not equal nrow(X).")
   }
   V
@@ -172,67 +180,65 @@ validate_sample_weights <- function(sample.weights, X) {
   }
 }
 
-#' @importFrom Matrix Matrix cBind
-#' @importFrom methods new
-create_train_matrices <- function(X, outcome = NULL, treatment = NULL,
-                                 instrument = NULL, censor = NULL,
-                                 sample.weights = FALSE) {
-  default.data <- matrix(nrow = 0, ncol = 0)
-  sparse.data <- new("dgCMatrix", Dim = c(0L, 0L))
+# Indices are offset by 1 for C++.
+create_train_matrices <- function(X,
+                                  outcome = NULL,
+                                  treatment = NULL,
+                                  instrument = NULL,
+                                  survival.numerator = NULL,
+                                  survival.denominator = NULL,
+                                  censor = NULL,
+                                  sample.weights = FALSE) {
   out <- list()
-  i <- 1
+  offset <- ncol(X) - 1
   if (!is.null(outcome)) {
-    out[["outcome.index"]] <- ncol(X) + i
+    out[["outcome.index"]] <- (offset + 1):(offset + NCOL(outcome))
+    offset <- offset + NCOL(outcome)
   }
   if (!is.null(treatment)) {
-    i <- i + 1
-    out[["treatment.index"]] <- ncol(X) + i
+    out[["treatment.index"]] <- (offset + 1):(offset + NCOL(treatment))
+    offset <- offset + NCOL(treatment)
   }
   if (!is.null(instrument)) {
-    i <- i + 1
-    out[["instrument.index"]] <- ncol(X) + i
+    out[["instrument.index"]] <- offset + 1
+    offset <- offset + 1
+  }
+  if (!is.null(survival.numerator)) {
+    out[["causal.survival.numerator.index"]] <- offset + 1
+    offset <- offset + 1
+  }
+  if (!is.null(survival.denominator)) {
+    out[["causal.survival.denominator.index"]] <- offset + 1
+    offset <- offset + 1
   }
   if (!is.null(censor)) {
-    i <- i + 1
-    out[["censor.index"]] <- ncol(X) + i
+    out[["censor.index"]] <- offset + 1
+    offset <- offset + 1
   }
-  if (!isFALSE(sample.weights)) {
-    i <- i + 1
-    out[["sample.weight.index"]] <- ncol(X) + i
+  # Forest bindings without sample weights: sample.weights = FALSE
+  # Forest bindings with sample weights:
+  # -sample.weights = NULL if no weights passed
+  # -sample.weights = numeric vector if passed
+  if (is.logical(sample.weights)) {
+    sample.weights <- NULL
+  } else {
+    out[["sample.weight.index"]] <- offset + 1
     if (is.null(sample.weights)) {
       out[["use.sample.weights"]] <- FALSE
     } else {
       out[["use.sample.weights"]] <- TRUE
     }
-  } else {
-    sample.weights = NULL
   }
 
-  if (inherits(X, "dgCMatrix") && ncol(X) > 1) {
-    sparse.data <- cbind(X, outcome, treatment, instrument, censor, sample.weights)
-  } else {
-    X <- as.matrix(X)
-    default.data <- as.matrix(cbind(X, outcome, treatment, instrument, censor, sample.weights))
-  }
-  out[["train.matrix"]] <- default.data
-  out[["sparse.train.matrix"]] <- sparse.data
+  X <- as.matrix(X)
+  out[["train.matrix"]] <- as.matrix(cbind(X, outcome, treatment, instrument, survival.numerator, survival.denominator, censor, sample.weights))
 
   out
 }
 
-#' @importFrom Matrix Matrix cBind
-#' @importFrom methods new
 create_test_matrices <- function(X) {
-  default.data <- matrix(nrow = 0, ncol = 0)
-  sparse.data <- new("dgCMatrix", Dim = c(0L, 0L))
   out <- list()
-  if (inherits(X, "dgCMatrix") && ncol(X) > 1) {
-    sparse.data <- X
-  } else {
-    default.data <- as.matrix(X)
-  }
-  out[["test.matrix"]] <- default.data
-  out[["sparse.test.matrix"]] <- sparse.data
+  out[["test.matrix"]] <- as.matrix(X)
 
   out
 }
@@ -241,7 +247,7 @@ observation_weights <- function(forest) {
   # Case 1: No sample.weights
   if (is.null(forest$sample.weights)) {
     if (length(forest$clusters) == 0 || !forest$equalize.cluster.weights) {
-      raw.weights <- rep(1, length(forest$Y.orig))
+      raw.weights <- rep(1, NROW(forest$Y.orig))
     } else {
       # If clustering with no sample.weights provided and equalize.cluster.weights = TRUE, then
       # give each observation weight 1/cluster size, so that the total weight of each cluster is the same.
@@ -272,3 +278,19 @@ do.call.rcpp = function(what, args, quote = FALSE, envir = parent.frame()) {
   names(args) = gsub("\\.", "_", names(args))
   do.call(what, args, quote, envir)
 }
+
+validate_subset <- function(forest, subset) {
+  if (is.null(subset)) {
+    subset <- 1:NROW(forest$Y.orig)
+  }
+  if (class(subset) == "logical" && length(subset) == NROW(forest$Y.orig)) {
+    subset <- which(subset)
+  }
+  if (!all(subset %in% 1:NROW(forest$Y.orig))) {
+    stop(paste(
+      "If specified, subset must be a vector contained in 1:n,",
+      "or a boolean vector of length n."
+    ))
+  }
+  subset
+}
